# add gems to $: by `gem` method
#  this is only way when install eye as system wide

gem 'celluloid',     '~> 0.16.0'
gem 'celluloid-io',  '~> 0.16.0'
gem 'nio4r'
gem 'timers'

gem 'state_machine'
gem 'sigar',         '~> 0.7.2'
